@@ -1,10 +1,18 @@
 # core Python packages
-import datetime, os, logging, itertools
+import datetime
+import itertools
+import logging
+import os
+import sys
 from copy import deepcopy
 
 
 # third party packages
 import pyhcup
+from pyhcup.parser import df_wtl
+from pyhcup.parser import replace_sentinels
+from pyhcup.db import insert_sql as gen_insert_sql
+from pyhcup.meta import is_long as meta_is_long
 
 
 # django packages
@@ -21,10 +29,6 @@ from djhcup_staging.models import State, DataSource, FileType, File, Column, Imp
 from djhcup_staging.utils.misc import dead_batches
 
 
-# pull these settings into local namespace
-from djhcup_core.defaults import DJHCUP_IMPORT_PATHS
-
-
 # start a logger
 logger = logging.getLogger('default')
 
@@ -214,7 +218,7 @@ def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
             meta = pyhcup.sas.meta_from_sas(load.file.loadfile.full_path)
             augmented = pyhcup.meta.augment(meta)
             
-            #store the columns for future reference
+            # store the columns for future reference
             for k, v in augmented.T.to_dict().iteritems():
                 #col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
                 col = Column(
@@ -231,9 +235,9 @@ def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
             meta = pyhcup.tx.meta_from_txt(load.file.loadfile.full_path)
             augmented = pyhcup.meta.augment(meta, dialect='PUDF')
             
-            #store the columns for future reference
+            # store the columns for future reference
             for k, v in augmented.T.to_dict().iteritems():
-                #col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
+                # col_supplemental_meta = pyhcup.db.col_from_invalue(v['informat'])
                 col = Column(
                     parent_file=load.file,
                     name=v['field'],
@@ -245,10 +249,10 @@ def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
                 col.save()
         
         if load.file.file_type.compression == 'NONE':
-            #uncompressed
+            # uncompressed
             handle = open(load.file.full_path)
         elif load.file.file_type.compression == 'ZIP':
-            #pkzip compression
+            # pkzip compression
             handle = pyhcup.hachoir.openz(os.path.dirname(load.file.full_path), load.file.filename)
         
         # make a generator to yield one slice_size slice of the file in question at a time
@@ -258,6 +262,7 @@ def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
         schema = destination.schema
         state = load.file.state.abbreviation
         year = load.file.year
+        dsabbr = destination.source.abbreviation
         
         
         # take 5
@@ -265,8 +270,15 @@ def import_file(self, importqueue_pk, insert_size=5, slice_size=1000):
         # this is preferred to a chain so that it doesn't wait to start until after all the signatures are generated
         result_lst = []
         for df in reader:
+            cleaned_df = replace_sentinels(df)
+            
+            if dsabbr == 'CHGS' and not meta_is_long(meta):
+                # this is a wide charges file
+                # convert before chunking it out
+                cleaned_df = df_wtl(cleaned_df, category='CHGS')
+            
             result_lst.append(chunker(
-                    df,
+                    cleaned_df,
                     table,
                     schema,
                     state,
@@ -316,19 +328,35 @@ def chunk_insert(df_chunk, table, schema, state, year, placeholder_representatio
         
     cursor = cnxns[HACHOIR_DB_ENTRY].cursor()
     
-    df_chunk = pyhcup.parser.replace_sentinels(df_chunk)
+    #df_chunk = replace_sentinels(df_chunk)
     
     try:
-        insert_sql, values = pyhcup.db.insert_sql(df_chunk, table=table, schema=schema, placeholder=placeholder_representation)
-        #logger.debug('Generated sql and values lists (params)')
-        cursor.execute(insert_sql, values)
-        #logger.debug('Executed sql using params')
-        
+        #print "df_chunk is %s" % df_chunk
+        #print "table is %s" % table
+        #print "schema is %s" % schema
+        #print "placeholder is %s" % placeholder_representation
+        insert_sql, values = gen_insert_sql(df_chunk, table, schema=schema, placeholder=placeholder_representation)    
+        logger.debug('Generated sql and values lists (params)')
     except:
         e = sys.exc_info()[0]
-        logger.warning('Failed to insert chunk; records from state-year %s-%s in table %s are likely incomplete and may need to be purged (%s)' % (state, year, table, e))
+        logger.warning('Failed to generate SQL insert statement and parameters list using pyhcup.db.insert_sql(). Raised %s.' % e)
+        logger.warning('df_chunk is %s' % df_chunk)
+        logger.warning('table is %s' % table)
+        logger.warning('schema is %s' % schema)
+        logger.warning('placeholder is %s' % placeholder_representation)
         return False
     
+    
+    try:
+        cursor.execute(insert_sql, values)
+        logger.debug('Executed sql using params')
+    except:
+        e = sys.exc_info()[0]
+        logger.warning('Failed to execute SQL insert statement and parameters list using cursor.execute(). Raised %s.' % e)
+        logger.warning('insert_sql is %s' % insert_sql)
+        logger.warning('values is %s' % values)
+        
+    
     tx.commit(using=HACHOIR_DB_ENTRY)
     #logger.debug('Committed transaction')
     
