@@ -1,5 +1,8 @@
 # core Python packages
-import datetime, os, logging
+import datetime
+import logging
+import os
+import sys
 
 
 # third party packages
@@ -13,6 +16,8 @@ from django.http import HttpResponse
 from django.utils.timezone import utc
 from django.conf import settings
 from django.core.urlresolvers import reverse
+from django.db import connections as cnxns
+from django.db import transaction as tx
 
 
 # local project and app packages
@@ -323,78 +328,88 @@ def file_match(request):
 
 
 def src_gen_master_table(request, source_id):
-    """Use all known loadfiles for source and generate one master table
-    
+    """For wide data, use all known loadfiles for source and generate one master table. For long data (or data which can be longed), make the corresponding fixed-style table.
     """
-
-    # import shortcuts to use with raw sql (!) later
-    from django.db import connections as cnxns
-    from django.db import transaction as tx
     
     source = DataSource.objects.get(pk=source_id)
     logger.info('Attempting to generate master table for %s' % source)
     
-    src_loadfiles = File.objects.filter(file_type__content_type__in=['LOAD'], file_type__source=source)
-    logger.info('Found %d loadfile records matching source' % len(src_loadfiles))
-    
-    # need later for table creation, but want the value to be based on the start of the process
-    now = datetime.datetime.utcnow().replace(tzinfo=utc).strftime("%Y%m%d%H%M%S")
-    
-    meta_list = []
-    excluded = []
-    for lf in src_loadfiles:
-        try:
-            if lf.state.abbreviation.lower() == 'tx':
-                # read the meta using the tx pudf stuff
-                mdf = pyhcup.tx.meta_from_txt(lf.full_path)
-                # mdf['state'] = 'tx'
-                # mdf['year']
-                augmented = pyhcup.meta.augment(mdf, dialect='pudf')
-            
-            else:
-                # read the meta using the sas stuff
-                mdf = pyhcup.sas.meta_from_sas(lf.full_path)
-                augmented = pyhcup.meta.augment(mdf)
-            
-            # in any case, put in the meta list collection
-            meta_list.append(augmented)
-            
-        except:
-            # failed to read these in for some reason; held here for debug
-            excluded.append(lf.full_path)
-    
-    merged_meta = pyhcup.meta.merge(meta_list)
-    logger.info('Merged %d meta DataFrames successfully, with %d columns after de-duplication' % (len(meta_list), len(merged_meta)))
-    
     cursor = cnxns[DJHCUP_DB_ENTRY].cursor()
+
+    # need later for table creation, but want the value to be based on the start of the process
+    now = datetime.datetime.utcnow().replace(tzinfo=utc).strftime("%Y%m%d%H%M%S")    
     db_table = "%s_%s" % (source.mr_descriptor, now)
-    constraints = ['NULL']
-    if source.top_level_source.abbreviation == 'HCUP':
-        pk_fields = ['year', 'state', 'key']
-    elif source.top_level_source.abbreviation == 'PUDF':
-        pk_fields = None
+    pk_fields = []
+    index_fields = []
     
-    create_sql = pyhcup.db.table_sql(merged_meta, db_table, pk_fields=pk_fields, ine=True, append_state=True, default_constraints=constraints)
+    # check to see whether this source has a LONG_MAPS entry, which would indicate it ought to be a long table
+    from pyhcup.parser import LONG_MAPS
+    if source.abbreviation in LONG_MAPS.keys():
+        logger.info('Match found in LONG_MAPS. This should be a fixed-style long table.')
+        create_sql = pyhcup.db.long_table_sql(db_table, str(source.abbreviation))
+        index_fields = ['KEY', 'VISITLINK', 'STATE', 'YEAR']
     
+    else:
+        logger.info('No match found in LONG_MAPS. This should be a dynamically-generated wide table.')
+        
+        src_loadfiles = File.objects.filter(file_type__content_type__in=['LOAD'], file_type__source=source)
+        logger.info('Found %d loadfile records matching source' % len(src_loadfiles))
+        
+        meta_list = []
+        excluded = []
+        for lf in src_loadfiles:
+            try:
+                if lf.state.abbreviation.lower() == 'tx':
+                    # read the meta using the tx pudf stuff
+                    mdf = pyhcup.tx.meta_from_txt(lf.full_path)
+                    # mdf['state'] = 'tx'
+                    # mdf['year']
+                    augmented = pyhcup.meta.augment(mdf, dialect='pudf')
+                
+                else:
+                    # read the meta using the sas stuff
+                    mdf = pyhcup.sas.meta_from_sas(lf.full_path)
+                    augmented = pyhcup.meta.augment(mdf)
+                
+                # in any case, put in the meta list collection
+                meta_list.append(augmented)
+                
+            except:
+                # failed to read these in for some reason; held here for debug
+                excluded.append(lf.full_path)
+        
+        merged_meta = pyhcup.meta.merge(meta_list)
+        logger.info('Merged %d meta DataFrames successfully, with %d columns after de-duplication' % (len(meta_list), len(merged_meta)))
+        
+        constraints = ['NULL']
+        if source.top_level_source.abbreviation == 'HCUP':# redundant # and source.abbreviation not in LONG_MAPS.keys():
+            pk_fields = ['year', 'state', 'key']
+            index_fields += pk_fields
+        
+        create_sql = pyhcup.db.table_sql(merged_meta, db_table, pk_fields=pk_fields, ine=True, append_state=True, default_constraints=constraints)
+        
     try:
         cursor.execute(create_sql)
+        print "cursor.execute() without error"
         tx.commit(using=DJHCUP_DB_ENTRY)
+        print "tx.commit() without error"
     except:
-        raise Exception("Failed to create master table with query %s" % create_sql)
+        e = sys.exc_info()[0]
+        raise Exception("Failed to create master table with query %s (%s)" % (create_sql, e))
     
     logger.info('Created master table %s with on database %s' % (db_table, DJHCUP_DB_ENTRY))
     
-    if pk_fields != None:
+    if index_fields != None:
         try:
-            for col in pk_fields:
-                index = pyhcup.db.index_sql(col, db_table, db_schema)
+            for col in index_fields:
+                index = pyhcup.db.index_sql(col, db_table)
                 cursor.execute(index)
                 tx.commit(using=DJHCUP_DB_ENTRY)
-                logger.info('Created index for column %s with on table %s in schema %s' % (col, db_table, db_schema))
+                logger.info('Created index for column %s with on table %s' % (col, db_table))
         except:
             raise Exception("Failed to create indexes on table, particularly one that should have resulted from this query: %s" % index)
     
-    table_record = ImportTable(source=source, name=db_table, schema=db_schema, database=DJHCUP_DB_ENTRY)
+    table_record = ImportTable(source=source, name=db_table, database=DJHCUP_DB_ENTRY, schema=settings[DJHCUP_DB_ENTRY]['schema'])
     table_record.save()
     logger.info('Saved master table details as ImportTable record %d' % table_record.pk)
     
